test_name "Dashboard Rake Tasks"

require File.join(File.dirname(__FILE__), '../helpers/dashboard_rake_tasks_helper')

dashboard_hosts = get_dashboard_hosts

step "Verify there's at least one dashboard host" do
  assert(dashboard_hosts.length > 0, "No dashboard hosts found!")
end

#######################################################
# Parameters as environment variables deprecation test
#######################################################

step "test parameters as environment variables deprecation" do
  dashboard_hosts.each do |dashboard|
    results = run_rake_task_raw(dashboard, "nodeclass:list match=foo")
    stdout = results.stdout.split("\n")
    stderr = results.stderr.split("\n")

    assert(stderr.include?("Warning: Passing rake task parameters as environment variables is deprecated, use task arguments instead, e.g. \"rake some:task[param1,param2]\"."), "nodeclass:list should have shown a deprecation warning when using old parameter syntax")
  end
end

#################################
# `nodeclass` tasks
#################################

step "test nodeclass:add, nodeclass:list, nodeclass:del rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodeclass:add['fooclass']")
    results = run_rake_task(dashboard, "nodeclass:list")
    assert(results.include?("fooclass"), "nodeclass:add should have added 'fooclass'!  nodeclass:list returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodeclass:del['fooclass']")
    results = run_rake_task(dashboard, "nodeclass:list")
    assert((! results.include?("fooclass")), "nodeclass:del should have removed 'fooclass'!  nodeclass:list returned: #{results.join("\n")}")
  end
end

#################################
# `nodegroup` tasks
#################################

step "test nodegroup:add, nodegroup:list, nodegroup:del rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add['foogroup']")
    results = run_rake_task(dashboard, "nodegroup:list")
    assert(results.include?("foogroup"), "nodegroup:add should have added 'foogroup'!  nodegroup:list returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del['foogroup']")
    results = run_rake_task(dashboard, "nodegroup:list")
    assert((! results.include?("foogroup")), "nodegroup:del should have removed 'foogroup'!  nodegroup:list returned: #{results.join("\n")}")
  end
end


step "test nodegroup:addclass, nodegroup:listclasses, nodegroup:delclass rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add['foogroup'] nodeclass:add['fooclass'] nodeclass:add['barclass'] nodegroup:addclass['foogroup','fooclass'] nodegroup:addclass['foogroup','barclass']")

    results = run_rake_task(dashboard, "nodegroup:listclasses['foogroup']")
    assert(results.include?("fooclass"), "nodegroup:addclass should have added 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert(results.include?("barclass"), "nodegroup:addclass should have added 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:delclass['foogroup','fooclass'] nodegroup:delclass['foogroup','barclass']")

    results = run_rake_task(dashboard, "nodegroup:listclasses['foogroup']")
    assert((! results.include?("fooclass")), "nodegroup:delclass should have removed 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert((! results.include?("barclass")), "nodegroup:delclass should have removed 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del['foogroup'] nodeclass:del['fooclass'] nodeclass:del['barclass']")
  end
end


step "test nodegroup:edit rake task" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add['foogroup'] nodeclass:add['fooclass'] nodeclass:add['barclass'] nodegroup:edit['foogroup','fooclass\\,barclass']")

    results = run_rake_task(dashboard, "nodegroup:listclasses['foogroup']")
    assert(results.include?("fooclass"), "nodegroup:edit should have added 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert(results.include?("barclass"), "nodegroup:edit should have added 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:edit['foogroup','']")

    results = run_rake_task(dashboard, "nodegroup:listclasses['foogroup']")
    assert((! results.include?("fooclass")), "nodegroup:edit should have removed 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert((! results.include?("barclass")), "nodegroup:edit should have removed 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del['foogroup'] nodeclass:del['fooclass'] nodeclass:del['barclass']")
  end
end


step "test nodegroup:add_all_nodes rake task" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add['foogroup'] node:add['foo.localdomain'] node:add['bar.localdomain']")

    run_rake_task(dashboard, "nodegroup:add_all_nodes['foogroup']")

    results = run_rake_task(dashboard, "node:listgroups['foo.localdomain']")
    assert(results.include?("foogroup"), "nodegroup:add_all_nodes should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "node:listgroups['bar.localdomain']")
    assert(results.include?("foogroup"), "nodegroup:add_all_nodes should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del['foo.localdomain'] node:del['bar.localdomain'] nodegroup:del['foogroup']")
  end
end


step "test nodegroup:addgroup, nodegroup:listgroups, nodegroup:delgroup" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add['parentgroup'] nodegroup:add['childgroup'] nodegroup:addgroup['parentgroup','childgroup']")

    results = run_rake_task(dashboard, "nodegroup:listgroups['parentgroup']")
    assert(results.include?("childgroup"), "nodegroup:addgroup should have added 'childgroup'!  nodegroup:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:delgroup['parentgroup','childgroup']")

    results = run_rake_task(dashboard, "nodegroup:listgroups['parentgroup']")
    assert(!results.include?("childgroup"), "nodegroup:delgroup should have removed 'childgroup'!  nodegroup:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del['parentgroup'] nodegroup:del['childgroup']")
  end
end


step "test nodegroup:addclassparam, nodegroup:delclassparam, nodegroup:listclassparams rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add['foogroup'] nodeclass:add['fooclass'] nodegroup:addclass['foogroup','fooclass'] nodegroup:addclassparam['foogroup','fooclass','fooparam','foovalue'] nodegroup:addclassparam['foogroup','fooclass','barparam','barvalue']")

    results = run_rake_task(dashboard, "nodegroup:listclasses['foogroup']")
    assert(results.include?("fooclass"), "nodegroup:addclass should have added 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "nodegroup:listclassparams['foogroup','fooclass']")
    assert(results.include?("fooparam = \"foovalue\""), "nodegroup:addclassparam should have added 'fooclass = \"fooparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")
    assert(results.include?("barparam = \"barvalue\""), "nodegroup:addclassparam should have added 'barclass = \"barparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:delclassparam['foogroup','fooclass','fooparam'] nodegroup:delclassparam['foogroup','fooclass','barparam']")

    results = run_rake_task(dashboard, "nodegroup:listclassparams['foogroup','fooclass']")
    assert((! results.include?("fooparam = \"foovalue\"")), "nodegroup:delclassparam should have removed 'fooclass = \"fooparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")
    assert((! results.include?("barparam = \"barvalue\"")), "nodegroup:delclassparam should have removed 'barclass = \"barparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del['foogroup'] nodeclass:del['fooclass']")
  end
end


#################################
# `node` tasks
#################################

step "test node:add, node:list, node:del rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add['foo.localdomain']")
    results = run_rake_task(dashboard, "node:list")
    assert(results.include?("foo.localdomain"), "node:add should have added 'foo.localdomain'!  node:list returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del['foo.localdomain']")
    results = run_rake_task(dashboard, "node:list")
    assert((! results.include?("foo.localdomain")), "node:del should have removed 'foo.localdomain'!  node:list returned: #{results.join("\n")}")
  end
end

step "test node:classes, node:listclasses rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add['foo.localdomain'] nodeclass:add['fooclass'] nodeclass:add['barclass'] node:classes['foo.localdomain','fooclass\\,barclass']")

    results = run_rake_task(dashboard, "node:listclasses['foo.localdomain']")
    assert(results.include?("fooclass"), "node:classes should have added 'fooclass'!  node:listclasses returned: #{results.join("\n")}")
    assert(results.include?("barclass"), "node:classes should have added 'barclass'!  node:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:classes['foo.localdomain','']")

    results = run_rake_task(dashboard, "node:listclasses['foo.localdomain']")
    assert((! results.include?("fooclass")), "node:classes should have removed 'fooclass'!  node:listclasses returned: #{results.join("\n")}")
    assert((! results.include?("barclass")), "node:classes should have removed 'barclass'!  node:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del['foo.localdomain'] nodeclass:del['fooclass'] nodeclass:del['barclass']")
  end
end

step "test node:addclassparam, node:delclassparam, node:listclassparams rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add['foo.localdomain'] nodeclass:add['fooclass'] node:classes['foo.localdomain','fooclass'] node:addclassparam['foo.localdomain','fooclass','fooparam','foovalue'] node:addclassparam['foo.localdomain','fooclass','barparam','barvalue']")

    results = run_rake_task(dashboard, "node:listclasses['foo.localdomain']")
    assert(results.include?("fooclass"), "node:classes should have added 'fooclass'!  node:listclasses returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "node:listclassparams['foo.localdomain','fooclass']")
    assert(results.include?("fooparam = \"foovalue\""), "node:addclassparam should have added 'fooclass = \"fooparam\"'! node:listclassparams returned: #{results.join("\n")}")
    assert(results.include?("barparam = \"barvalue\""), "node:addclassparam should have added 'barclass = \"barparam\"'! node:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:delclassparam['foo.localdomain','fooclass','fooparam'] node:delclassparam['foo.localdomain','fooclass','barparam']")

    results = run_rake_task(dashboard, "node:listclassparams['foo.localdomain','fooclass']")
    assert((! results.include?("fooparam = \"foovalue\"")), "node:delclassparam should have removed 'fooclass = \"fooparam\"'! node:listclassparams returned: #{results.join("\n")}")
    assert((! results.include?("barparam = \"barvalue\"")), "node:delclassparam should have removed 'barclass = \"barparam\"'! node:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del['foo.localdomain'] nodeclass:del['fooclass']")
  end
end

step "test node:groups rake task" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add['foo.localdomain'] nodegroup:add['foogroup'] nodegroup:add['bargroup'] node:groups['foo.localdomain','foogroup\\,bargroup']")

    results = run_rake_task(dashboard, "node:listgroups['foo.localdomain']")
    assert(results.include?("foogroup"), "node:groups should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")
    assert(results.include?("bargroup"), "node:groups should have added 'bargroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:groups['foo.localdomain','']")

    results = run_rake_task(dashboard, "node:listgroups['foo.localdomain']")
    assert((! results.include?("foogroup")), "node:groups should have removed 'foogroup'!  node:listgroups returned: #{results.join("\n")}")
    assert((! results.include?("bargroup")), "node:groups should have removed 'bargroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del['foo.localdomain'] nodegroup:del['foogroup'] nodegroup:del['bargroup']")
  end
end

#########################################################
# `node` and `nodegroup` :parameters and :variables tasks
#########################################################

["node", "nodegroup"].each do |namespace|
  ["variables", "parameters"].each do |task|

    full_task = "#{namespace}:#{task}"
    step "test #{full_task} rake tasks" do
      dashboard_hosts.each do |dashboard|

        run_rake_task(dashboard, "#{namespace}:add['foo'] #{full_task}['foo','foovar=fooval\\,barvar=barval\\,letters=a\\\\\\,b\\\\\\,c\\,pet=cat\\\\=socks']")

        results = run_rake_task(dashboard, "#{full_task}['foo']")

        assert(results.include?("foovar=fooval"), "#{full_task} should have added 'foovar=fooval'! #{full_task}['foo'] returned: #{results.join("\n")}")
        assert(results.include?("barvar=barval"), "#{full_task} should have added 'barvar=barval'! #{full_task}['foo'] returned: #{results.join("\n")}")
        assert(results.include?("letters=a,b,c"), "#{full_task} should have added 'letters=a,b,c'! #{full_task}['bar'] returned: #{results.join("\n")}")
        assert(results.include?("pet=cat=socks"), "#{full_task} should have added 'pet=cat=socks'! #{full_task}['bar'] returned: #{results.join("\n")}")

        if task == "parameters"
          assert(results.include?("The #{namespace}:parameters task is deprecated. Please use the #{namespace}:variables task instead"),
                 "#{namespace}:parameters should have stated 'The #{namespace}:parameters task is deprecated. Please use the #{namespace}:variables task instead'! #{namespace}:parameters['foo'] returned: #{results.join("\n")}")
        end

        run_rake_task(dashboard, "#{namespace}:del['foo']")
      end
    end
  end
end
